<?php
namespace Ems\portalGestor\Repository;

use Ems\portalGestor\Service\OperadorService;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

class OperadorRepository extends Crud {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getDadosOperador(
        string $cdOperador,
        string $dtNascPessoa        = null,
        string $dtAdmissaoVinc      = null,
        string $cpfPessoa           = null,
        string $idConfirmrecsenport = null
    ) {
        try {
            $sql = <<<SQL
                SELECT
                    OP.CDOPERADOR,
                    OP.NMOPERADOR,
                    OP.NRPARCNEGOCIO,
                    OP.NRORG,
                    VM.NRVINCULOM,
                    VM.NRTIPOVINCULOM,
                    VM.DTRESCISAOVINC,
                    VH.NRESTRUTLEGAL,
                    VH.NRESTRUTGEREN,
                    EH.NMESCALATRABH,
                    ESH.NMRAZSOCESTRUT,
                    POPER.IDPGRESTRINGEREGIST,

                    /* PARAMETROS DA ORG QUE CONTROLAM A POSSIBILIDADE DE EDIÇÃO DE DADOS NO PORTAL */
                    PO.IDALTERPTOVINCULO,
                    PO.IDAPROVOCORRENCIA,
                    PO.IDALTRESUMOFREQ,
                    PO.IDTROCAHORARIO,
                    PO.IDCONFIRMRECSENPORT,
                    PO.IDREALIZAADMISSAO,
                    PO.IDREALIZACADAUTOM,
                    PO.IDDPALTERPTOVINCULO
                FROM
                    OPERADOR OP
                    JOIN GPE_PARAMORG      PO ON PO.NRORG = OP.NRORG
                    JOIN GPE_PARAMOPER POPER ON POPER.NRORG = OP.NRORG 
                        AND POPER.CDOPERADOR = OP.CDOPERADOR
                    /* JOIN DE PESSOA VIA SELECT POR MOTIVOS DE CUSTO (LEFT JOIN GPE_PESSOAH) */
                    LEFT JOIN (
                        SELECT
                            P.NRORG,
                            P.NRPESSOA,
                            P.NRPARCNEGOCIO,
                            PH.NRPESSOAH,
                            PH.NMPESSOA,
                            PH.DTNASCPESSOA,
                            PH.NRCPFPESSOA
                        FROM GPE_PESSOA P
                        JOIN GPE_PESSOAH PH ON PH.NRORG = P.NRORG AND PH.NRPESSOA = P.NRPESSOA AND PH.DTMESCOMPETENC = (
                            SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH WHERE NRORG = PH.NRORG AND NRPESSOA = PH.NRPESSOA
                        )
                    ) PESSOA ON PESSOA.NRORG = OP.NRORG AND PESSOA.NRPARCNEGOCIO = OP.NRPARCNEGOCIO
                    LEFT JOIN GPE_VINCULOM      VM ON VM.NRORG = OP.NRORG AND VM.NRPESSOA = PESSOA.NRPESSOA
                                                AND VM.DTADMISSAOVINC <= TRUNC(SYSDATE)
                                                AND (VM.DTRESCISAOVINC >= TRUNC(SYSDATE) OR VM.DTRESCISAOVINC IS NULL)
                    LEFT JOIN GPE_VINCULOH      VH ON VH.NRORG = OP.NRORG
                                            AND VH.NRVINCULOM = VM.NRVINCULOM
                                            AND VH.DTMESCOMPETENC = (
                        SELECT
                            MAX(VHH.DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH VHH
                        WHERE
                            VHH.NRORG = VH.NRORG
                            AND VHH.NRVINCULOM = VH.NRVINCULOM
                    )
                    LEFT JOIN ESTRUTURAM        ESM ON ESM.NRORG = OP.NRORG
                                        AND ESM.NRESTRUTURAM = VH.NRESTRUTGEREN
                    LEFT JOIN ESTRUTURAH        ESH ON ESH.NRORG = OP.NRORG
                                        AND ESH.NRESTRUTURAM = VH.NRESTRUTGEREN
                                        AND ESH.DTMESCOMPETENC = (
                        SELECT
                            MAX(ESHH.DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH ESHH
                        WHERE
                            ESHH.NRORG = ESH.NRORG
                            AND ESHH.NRESTRUTURAM = VH.NRESTRUTGEREN
                    )
                    LEFT JOIN GPE_ESCALATRABM   EM ON EM.NRORG = OP.NRORG
                                            AND EM.NRESCALATRABM = VH.NRESCALATRABM
                    LEFT JOIN GPE_ESCALATRABH   EH ON EM.NRESCALATRABM = EH.NRESCALATRABM
                                            AND EM.NRORG = EH.NRORG
                                            AND EH.IDATIVO = 'S'
                                            AND EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG
                            AND NRESCALATRABM = EH.NRESCALATRABM
                            AND DTMESCOMPETENC <= SYSDATE
                            AND ( EM.DTFIMVIGENCIA >= SYSDATE
                                OR EM.DTFIMVIGENCIA IS NULL )
                    )

                WHERE
                    OP.CDOPERADOR = :P_CDOPERADOR

                    /* VALIDATIONS IF PROVIDED */
                    AND (TO_CHAR(VM.DTADMISSAOVINC, 'DD/MM/YYYY')   = :P_DTADMISSAOVINC         OR :P_DTADMISSAOVINC        IS NULL)
                    AND (TO_CHAR(PESSOA.DTNASCPESSOA, 'DD/MM/YYYY') = :P_DTNASCPESSOA           OR :P_DTNASCPESSOA          IS NULL)
                    AND (PESSOA.NRCPFPESSOA                         = :P_NRCPFPESSOA            OR :P_NRCPFPESSOA           IS NULL)
                    AND (PO.IDCONFIRMRECSENPORT                     = :P_IDCONFIRMRECSENPORT    OR :P_IDCONFIRMRECSENPORT   IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_CDOPERADOR"          => $cdOperador,
                    "P_DTADMISSAOVINC"      => $dtAdmissaoVinc,
                    "P_DTNASCPESSOA"        => $dtNascPessoa,
                    "P_NRCPFPESSOA"         => $cpfPessoa,
                    "P_IDCONFIRMRECSENPORT" => $idConfirmrecsenport,
                )
            );
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLogin(string $nrvinculom, string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    1
                FROM
                    GPE_VINCULOM VM
                WHERE
                    VM.NRORG = :P_NRORG
                    and VM.NRVINCULOM = :P_NRVINCULOM
                    and VM.DTADMISSAOVINC <= TRUNC(SYSDATE)
                    AND (VM.DTRESCISAOVINC >= TRUNC(SYSDATE) OR VM.DTRESCISAOVINC IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRVINCULOM" => $nrvinculom,
                    "P_NRORG" => $nrorg
                )
            );
            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNrParcnegocioOperador(string $cdoperador, int $nrorg): int {
        $sql = <<<SQL
            SELECT NRPARCNEGOCIO
            FROM OPERADOR
            WHERE CDOPERADOR = :cdoperador
              AND NRORG = :nrorg
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                ":cdoperador" => $cdoperador,
                "nrorg" => $nrorg
            )
        );

        return $query->fetchAll();
    }

    public function getNrEstrutRespLider(string $cdoperador, string $nrorg): string {
        $sql = <<<SQL
            SELECT LISTAGG(NRESTRUTURAMREL, ', ') WITHIN GROUP (ORDER BY NRESTRUTURAMREL) ESTRUTURAS
                  FROM HIERARQUIAORG
                 WHERE NRORG = :P_NRORG
                   AND (NRESTRUTURAMSUP IN (SELECT
                                        ESTRUTURAM.NRESTRUTURAM
                                    FROM ESTRUTURAM
                                    JOIN ESTRUTURAH
                                        ON ESTRUTURAH.NRESTRUTURAM  = ESTRUTURAM.NRESTRUTURAM
                                        AND ESTRUTURAH.NRORG         = ESTRUTURAM.NRORG
                                    JOIN PARCNEGOCIO
                                        ON PARCNEGOCIO.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
                                        AND PARCNEGOCIO.NRORG         = ESTRUTURAM.NRORG
                                    JOIN OPERADOR
                                        ON OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                        AND OPERADOR.NRORG         = PARCNEGOCIO.NRORG
                                    WHERE ESTRUTURAM.NRORG = :P_NRORG
                                        AND (OPERADOR.CDOPERADOR = LPAD(:P_CDOPERADOR, 12, 0) OR OPERADOR.CDOPERADOR = :P_CDOPERADOR))
                                        OR NRESTRUTURAMREL IN (SELECT
                                        ESTRUTURAM.NRESTRUTURAM
                                    FROM ESTRUTURAM
                                    JOIN ESTRUTURAH
                                        ON ESTRUTURAH.NRESTRUTURAM  = ESTRUTURAM.NRESTRUTURAM
                                        AND ESTRUTURAH.NRORG         = ESTRUTURAM.NRORG
                                    JOIN PARCNEGOCIO
                                        ON PARCNEGOCIO.NRPARCNEGOCIO = ESTRUTURAH.NRPARCRESPINF
                                        AND PARCNEGOCIO.NRORG         = ESTRUTURAM.NRORG
                                    JOIN OPERADOR
                                        ON OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                        AND OPERADOR.NRORG         = PARCNEGOCIO.NRORG
                                    WHERE ESTRUTURAM.NRORG = :P_NRORG
                                        AND (OPERADOR.CDOPERADOR = LPAD(:P_CDOPERADOR, 12, 0) OR OPERADOR.CDOPERADOR = :P_CDOPERADOR)))
            ORDER BY NRESTRUTURAMREL
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute(
            array(
                ":P_CDOPERADOR" => $cdoperador,
                ":P_NRORG" => $nrorg
            )
        );

        $result = $query->fetch()['ESTRUTURAS'];

        return $result;
    }

    public function getClientSecret(array $params): array {
        $sql = <<<SQL
            SELECT CLIENT_SECRET
            FROM ORGANIZACAO
            WHERE NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();

        return $result;
    }

    public function getPerfisDeAcesso(array $params): array {
        $sql = <<<SQL
            SELECT PERFPARC.NRPERFILACESSO
            FROM PERFACESSPARC PERFPARC
            WHERE PERFPARC.NRPARCNEGOCIO = (
                SELECT OPER.NRPARCNEGOCIO
                FROM OPERADOR OPER
                WHERE OPER.CDOPERADOR = :P_CDOPERADOR
                  AND OPER.NRORG = PERFPARC.NRORG
            )
              AND PERFPARC.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $perfis = $std->fetchAll();

        return $perfis;
    }

    public function getVinculoCdOperador(array $params) {
        $sql = <<<SQL
            SELECT OPER.CDOPERADOR
            FROM GPE_VINCULOH VM
            JOIN GPE_PESSOA PE ON
                    PE.NRPESSOA = VM.NRPESSOA
                AND PE.NRORG = VM.NRORG
            JOIN OPERADOR OPER ON
                    OPER.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                AND OPER.NRORG = PE.NRORG
            WHERE OPER.CDOPERADOR = :P_CDOPERADOR
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $cdOperador = $std->fetch();

        return $cdOperador;
    }

    public function getBeforeForgotPasswordInfo(array $params) {
        $sql = <<<SQL
            SELECT
                OP.NRORG,
                OP.CDOPERADOR,
                OP.DSEMAILOPER,
                OP.NMACESSO,
                PO.IDCONFIRMRECSENPORT AS SHOW_CONFIRMATION_RESET
            FROM OPERADOR OP
            JOIN GPE_PARAMORG PO ON PO.NRORG = OP.NRORG
            WHERE UPPER(OP.NMACESSO) = UPPER(:P_DSEMAILOPER) /* LOGIN OPERATOR SEARCH */
                OR OP.DSEMAILOPER = :P_DSEMAILOPER
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $dadosOperador = $std->fetch();

        return empty($dadosOperador) ? [] : $dadosOperador;
    }

    public function clearOperatorPassword(array $params) {
        $sql = <<<SQL
            UPDATE OPERADOR
            SET CDSENHAOPERWEB = :P_CDSENHAOPERWEB
            WHERE CDOPERADOR = :P_CDOPERADOR AND NRORG = :P_NRORG
SQL;

        $params['P_CDSENHAOPERWEB'] = OperadorService::OPERATOR_DEFAULT_HASH;

        $std = $this->connection->prepare($sql);
        return $std->execute($params);
    }

    public function getOperadorByNrParcNegocio(
        int $nrOrg,
        int $nrParcNegocio
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                OPERADOR
            WHERE
                NRORG = :P_NRORG AND
                NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $params =[
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetch();
        return $result;
    }
}